/*
Copyright (c) 2005 Andrew Olney

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
associated documentation files (the "Software"), to deal in the Software without restriction, 
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial 
portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN 
NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
using System;

namespace LSAModule
{
	/// <summary>
	/// ILSACalculator specifies an interface to functions that are automatically 
	/// generated by RewriteMatrix in a highly optimized way. This level of implementation 
	/// deals directly with vector calculations.  String to string calculations are handled
	/// by the Semantic Analyzer
	/// </summary>
	public interface ILSACalculator
	{
		//The major players
		float Cosine( float[] aVector, float[] bVector );
		float Length( float[] aVector );
		float DotProduct( float[] aVector, float[] bVector );
		float[] Normalized( float[] aVector );
		float[] ElementWiseAddition( float[] aVector, float[] bVector );
		float[] ElementWiseSubtraction( float[] aVector, float[] bVector );
		float[] ElementWiseMultiplication( float[] aVector, float[] bVector );
		float[] ElementWiseDivision( float[] aVector, float[] bVector );
		float[] ScalarMultiplication( float[] aVector, float scalar );

		//Couldn't resist
		float[] ParallelVectorToSpan( float[] aVector, System.Collections.ArrayList span );
	}
}
